@@ -4,18 +4,23 @@ import logging
 import requests
 import copy
 import re
+from urllib.parse import urlparse
 from lxml import html
 from typing import Optional, List
 from findpapers.models.search import Search
 from findpapers.models.paper import Paper
 from findpapers.models.publication import Publication
+from findpapers.utils.requests_util import DefaultSession
 import findpapers.searchers.scopus_searcher as scopus_searcher
 import findpapers.searchers.ieee_searcher as ieee_searcher
 import findpapers.searchers.pubmed_searcher as pubmed_searcher
 import findpapers.searchers.arxiv_searcher as arxiv_searcher
 import findpapers.searchers.acm_searcher as acm_searcher
+import findpapers.searchers.medrxiv_searcher as medrxiv_searcher
+import findpapers.searchers.biorxiv_searcher as biorxiv_searcher
 import findpapers.utils.common_util as common_util
 import findpapers.utils.persistence_util as persistence_util
+import findpapers.utils.publication_util as publication_util
 
 
 def _get_paper_metadata_by_url(url: str):
@@ -140,9 +145,16 @@ def _enrich(search: Search, scopus_api_token: Optional[str] = None):
                     if paper_authors is not None and len(paper_authors) > 0:
                         paper.authors = paper_authors
 
-                    paper_keywords = _force_single_metadata_value_by_key(paper_metadata, 'keywords')
+                    paper_keywords = _force_single_metadata_value_by_key(paper_metadata, 'citation_keywords')
+                    if paper_keywords is None or len(paper_keywords.strip()) > 0:
+                        paper_keywords = _force_single_metadata_value_by_key(paper_metadata, 'keywords')
+
                     if paper_keywords is not None and len(paper_keywords.strip()) > 0:
-                        paper_keywords = set(paper_keywords.split(','))
+                        if ',' in paper_keywords:
+                            paper_keywords = paper_keywords.split(',')
+                        elif ';' in paper_keywords:
+                            paper_keywords = paper_keywords.split(';')
+                        paper_keywords = set([x.strip() for x in paper_keywords])
 
                     if paper_keywords is not None and len(paper_keywords) > 0:
                         paper.keywords = paper_keywords
@@ -160,7 +172,7 @@ def _enrich(search: Search, scopus_api_token: Optional[str] = None):
                         publication_title = _force_single_metadata_value_by_key(paper_metadata, 'citation_book_title')
                         publication_category = 'Book'
 
-                    if publication_title is not None and len(publication_title) > 0:
+                    if publication_title is not None and len(publication_title) > 0 and publication_title.lower() not in ['biorxiv', 'medrxiv', 'arxiv']:
                     
                         publication_issn = _force_single_metadata_value_by_key(paper_metadata, 'citation_issn')
                         publication_isbn = _force_single_metadata_value_by_key(paper_metadata, 'citation_isbn')
@@ -210,6 +222,39 @@ def _filter(search: Search):
                 pass
 
 
+def _flag_potentially_predatory_publications(search: Search):
+    """
+    Flag all the potentially predatory publications
+
+    Parameters
+    ----------
+    search : Search
+        A search instance
+    """
+
+    for paper in list(search.papers):
+        try:
+
+            if paper.publication is not None:
+                publication_name = paper.publication.title.lower()
+                publication_host = None
+            
+                if paper.doi is not None:
+                    url = f'http://doi.org/{paper.doi}'
+                    response = common_util.try_success(lambda url=url: DefaultSession().get(url), 2)
+
+                    if response is not None:
+                        publication_host = urlparse(response.url).netloc.replace("www.", "")
+
+                if publication_name in publication_util.POTENTIAL_PREDATORY_JOURNALS_NAMES \
+                    or publication_host in publication_util.POTENTIAL_PREDATORY_JOURNALS_HOSTS:
+
+                    paper.publication.is_potentially_predatory = True
+
+        except Exception:
+            pass
+
+
 def _database_safe_run(function: callable, search: Search, database_label: str):
     """
     Private method that calls a provided function catching all exceptions without rasing them, only logging a ERROR message
@@ -460,6 +505,14 @@ def search(outputpath: str, query: Optional[str] = None, since: Optional[datetim
     else:
         logging.info('Scopus API token not found, skipping search on this database')
 
+    if databases is None or medrxiv_searcher.DATABASE_LABEL.lower() in databases:
+        _database_safe_run(lambda: medrxiv_searcher.run(search),
+                        search, medrxiv_searcher.DATABASE_LABEL)
+
+    if databases is None or biorxiv_searcher.DATABASE_LABEL.lower() in databases:
+        _database_safe_run(lambda: biorxiv_searcher.run(search),
+                        search, biorxiv_searcher.DATABASE_LABEL)
+
     logging.info('Enriching results...')
 
     _enrich(search, scopus_api_token)
@@ -472,6 +525,10 @@ def search(outputpath: str, query: Optional[str] = None, since: Optional[datetim
 
     search.merge_duplications()
 
+    logging.info('Flagging potentially predatory publications...')
+
+    _flag_potentially_predatory_publications(search)
+
     logging.info(f'It\'s finally over! {len(search.papers)} papers retrieved. Good luck with your research :)')
 
     persistence_util.save(search, outputpath)
